<?php

namespace App\Exceptions;

use App\Common\Exceptions\AjaxException;
use App\Common\Exceptions\ApiAjaxException;
use App\Common\Exceptions\ApiException;
use Exception;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Illuminate\Validation\ValidationException;
use Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class Handler extends ExceptionHandler
{
    /**
     * A list of the exception types that should not be reported.
     *
     * @var array
     */
    protected $dontReport = [
        \Illuminate\Auth\AuthenticationException::class,
        \Illuminate\Auth\Access\AuthorizationException::class,
        \Symfony\Component\HttpKernel\Exception\HttpException::class,
        \Illuminate\Database\Eloquent\ModelNotFoundException::class,
        \Illuminate\Session\TokenMismatchException::class,
        \Illuminate\Validation\ValidationException::class,
    ];

    /**
     * Report or log an exception.
     *
     * This is a great spot to send exceptions to Sentry, Bugsnag, etc.
     *
     * @param  \Exception  $exception
     * @return void
     */
    public function report(Exception $exception)
    {
        parent::report($exception);
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Exception  $exception
     * @return \Illuminate\Http\Response
     */
    public function render($request, Exception $exception)
    {
        if ($exception instanceof ValidationException){
            $request = app()->make('request');
            if ($request->ajax()) {
                return response()->json([
                    'errno' => $exception->status,
                    'data' => [],
                    'message' => $exception->validator->errors()->first()
                ]);
            }
        }

        if ($exception instanceof MethodNotAllowedHttpException){
            return response()->json([
                'errno' => 405,
                'data' => [],
                'message' => 'Method Not Allowed'
            ]);
        }

        if ($exception instanceof NotFoundHttpException){
            return response()->json([
                'errno' => 404,
                'data' => [],
                'message' => 'Not Found'
            ]);
        }

        if ($exception instanceof ApiException
        && env('APP_DEBUG')
        ){
            return response()->json([
                'status' => $exception->getCode(),
                'data' => [],
                'msg' => $exception->getMessage()
            ], 200, ['charset' => 'utf-8'])->send();
        }

        return parent::render($request, $exception);
    }

    /**
     * Convert an authentication exception into an unauthenticated response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Illuminate\Auth\AuthenticationException  $exception
     * @return \Illuminate\Http\Response
     */
    protected function unauthenticated($request, AuthenticationException $exception)
    {
        if ($request->expectsJson()) {
            return response()->json(['error' => 'Unauthenticated.'], 401);
        }

        return redirect()->guest(route('login'));
    }
}
