<?php

namespace App\Models;

use Auth;
use Illuminate\Database\Eloquent\Model;
use App\Common\Exceptions\AjaxException;
use Illuminate\Support\Str;

class BaseModel extends Model
{
    const PAGE_SIZE = 20;

    /**
     * 通过参数获取查询
     * @param $query
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public static function search($query, $builder = null)
    {
        if (!$builder) {
            $static = self::query();
            $model = $oStatic = new static();
        } else {
            $static = $oStatic = $builder;
            $model = $oStatic->getModel();
        }

        if (!$query) {
            return $static;
        }

        foreach ($query as $column => $value) {
            if (!isNotNullAndNotEmptyString($value)) {
                continue;
            }
            $afterFun = Str::camel($column);
            $scopeMethod = 'scope' . ucfirst($afterFun);

            if (method_exists($model, $scopeMethod)) {
                $static = $static->$afterFun($value);
            } else {
                if (is_array($value)) {
                    $static = $static->whereIn($column, $value);
                } else {
                    if ($value !== '') {
                        $static = $static->where($column, $value);
                    }
                }
            }
        }

        return $static;
    }

    public static function instance()
    {
        static $instance;
        if (is_null($instance)) {
            $instance = new static;
        }
        return $instance;
    }

    /**
     * 内置过滤器
     */
    public function validateFilter($data, $senario)
    {
        return $data;
    }

    /**
     * 默认规则
     */
    public function validateRules($request)
    {
        return [
            'default' => [],
        ];
    }

    /*
     * 默认返回信息
     */
    public function validateMessage()
    {
        return ['default' => []];
    }

    /**
     * 获取验证规则
     */
    public function getRules($request, $scenario)
    {

        $defaultRules = $scenarioRules = [];
        $rules = $this->validateRules($request);

        if (isset($rules['default'])) {
            $defaultRules = $rules['default'];
        }
        if (isset($rules[$scenario])) {
            $scenarioRules = $rules[$scenario];
        }
        $defaultRules = array_merge($defaultRules, $scenarioRules);
        return $defaultRules;
    }

    /**
     * 获取验证提示信息
     */
    public function getMessages($scenario)
    {
        $defaultMessages = $scenarioMessages = [];
        $messages = $this->validateMessage();
        if (isset($messages['default'])) {
            $defaultMessages = $messages['default'];
        }
        if (isset($messages[$scenario])) {
            $scenarioMessages = $messages[$scenario];
        }
        $defaultMessages = array_merge($defaultMessages, $scenarioMessages);
        return $defaultMessages;
    }

    /**
     * 场景字段
     */
    public function getScenarios($scenario)
    {
        $defaultScenarios = $scenarios = [];
        if (isset($this->scenarios['default'])) {
            $defaultScenarios = $this->scenarios['default'];
        }
        if (isset($this->scenarios[$scenario])) {
            $scenarios = $this->scenarios[$scenario];
        }

        $defaultScenarios = array_merge($defaultScenarios, $scenarios);
        $defaultScenarios = array_unique($defaultScenarios);
        return $defaultScenarios;
    }

    /**
     * 获取显示的字段 可使用 makeVisible
     * @param Array $fields $this->fields 下标
     * @param String | Array $appends 追加显示
     * @return array
     */
    public function getFields($fields, $appends = [])
    {
        $data = $result = [];
        foreach ((array)$fields as $field) {
            $data = array_merge($data, $this->fields[$field]);
        }
        $data = array_merge($data, (array)$appends);
        $data = array_unique($data);
        foreach ($data as $key) {
            $result[$key] = $this->$key;
        }
        return $result;
    }

    public function loadModel($id, $message = '没有找到数据')
    {
        $model = static::find($id);
        if (!$model) {
            throw new AjaxException(value($message));
        }
        return $model;
    }

    /**
     * 通过data查询
     * @param $data
     */
    public static function be($data)
    {
        if (!$data) {
            return false;
        }

        $builder = self::query();

        foreach ($data as $column => $value) {
            $builder->where($column, $value);
        }

        return $builder->count() > 0;
    }


}
