<?php

/**
 * 商品
 */

namespace App\Models;

use \App\Modules\Live\Models\BaseModel;
use App\Observers\Goods\GoodsObserver;
use App\Services\AppService;
use App\Services\GlobalRoleService;
use Illuminate\Database\Eloquent\SoftDeletes;

class ShimmerLiveshopGoods extends BaseUniacidModel
{
    use SoftDeletes;

    protected $table = 'shimmer_liveshop_goods';
    protected $guarded = ['extends'];
    protected $casts = [
        'ensures' => 'json',
        'thirdurl' => 'json',
    ];
    public static $states = [
        1 => '上架',
        0 => '下架',
    ];

    public static function boot()
    {
        parent::boot();
        static::observe(new GoodsObserver);
    }

    /**
     * 保障
     * @var array
     */
    public static $ensures = [
        1 => '七天无理由退货',
        2 => '正品保障',
        3 => '售后无忧',
    ];

    /**
     * 通过商品名称搜索
     * @param $query
     * @param $nickname
     */
    public function scopeTitle($query, $title)
    {
        if (!$title) {
            return $query;
        }

        return $query->where('title', 'like', "%{$title}%");
    }

    /**
     * 商品上下架状态
     * @param $query
     * @param $state
     */
    public function scopeState($query, $state)
    {
        return $query->where('state', $state);
    }

    /**
     * 是否是精选商品
     * @param $query
     * @param $isVip
     */
    public function scopeIsVip($query, $isVip)
    {
        return $query->where('is_vip', $isVip);
    }

    /**
     * 是否包邮
     * @param $query
     * @param $isFreeFreight
     */
    public function scopeIsFreeFreight($query, $isFreeFreight)
    {
        return $query->where('is_free_freight', $isFreeFreight);
    }

    /**
     * 是否是首发单品
     * @param $query
     * @param $isNew
     */
    public function scopeIsNew($query, $isNew)
    {
        return $query->where('is_new', $isNew);
    }

    /**
     * 是否是热卖单品
     * @param $query
     * @param $isHot
     */
    public function scopeIsHot($query, $isHot)
    {
        return $query->where('is_hot', $isHot);
    }

    /**
     * 是否是促销商品
     * @param $query
     * @param $isHot
     */
    public function scopeIsSales($query, $isSales)
    {
        return $query->where('is_sales', $isSales);
    }

    public function getThumbAttribute($thumb)
    {
        if ($thumb) {
            $thumb = wgTomedia($thumb);
        }

        return $thumb;
    }

    public function getThumbsAttribute($thumbs)
    {
        if ($thumbs && !is_array($thumbs)) {
            $thumbs = explode(',', $thumbs);
            foreach ($thumbs as $key => $thumb) {
                $thumbs[$key] = wgTomedia($thumb);
            }
        }
        return $thumbs;
    }

    /**
     * 获取角色
     */
    public function getRoleAttribute()
    {
        return GlobalRoleService::getRole($this->role_type, $this->role_id);
    }

    /**
     * 获取商品保障文字
     */
    public function getEnsuresTextAttribute()
    {
        $result = '';

        foreach ((array)$this->ensures as $ensure) {
            $result .= self::$ensures[$ensure] . ' | ';
        }

        $result = rtrim($result, ' | ');

        return $result;
    }

    /**
     * 获取购买方式文本
     */
    public function getBuywayTextAttribute()
    {
        switch ($this->buyway) {
            case 1:
                return '店铺购买';
            case 2:
                return '外链购买';
        }
    }

    public function validateRules($request)
    {
        return [
            'default' => [
                'title' => 'required',
            ],
        ];
    }

    public function validateMessage()
    {
        return [
            'default' => [
                'title.required' => '商品名称不能为空',
            ],
        ];
    }

    /**
     * 获取商品分类
     */
    public function getCategoryIds()
    {
        return ShimmerLiveshopGoodsCategoryXref::where('goods_id', $this->id)
            ->get()->pluck('category_id')->all();
    }

    /**
     * 通过id获取有效商品信息
     * @param $id
     */
    public static function getValidGoodsById($id)
    {
        if (!$id) {
            return false;
        }

        return self::where('id', $id)->state(1)->first();
    }

    /**
     * 修改商品上下架状态
     * @param $id
     * @param $state
     */
    public static function changeState($id, $state)
    {
        $result = false;

        if (!$id) {
            return $result;
        }

        $update = [
            'state' => $state,
        ];
        if ($state == 1) {
            $update['stateon_at'] = date('Y-m-d H:i:s');
        }
        if (is_array($id)) {
            $result = self::whereIn('id', $id)->update($update);
        } else {
            $result = self::where('id', $id)->update($update);
        }

        return $result;
    }

    /**
     * 获取商品标签id数组
     * @param $goodsId
     */
    public static function getGoodsTagIds($goodsId)
    {
        if (!$goodsId) {
            return [];
        }
        return ShimmerLiveshopRelationTagXref::where('relation_type', 1)
            ->where('relation_id', $goodsId)
            ->get()
            ->pluck('tag_id')
            ->toArray();
    }

    /**
     * 获取标签商品列表
     * @param $tagId
     */
    public static function getTagGoods($tagId, $limit = 0)
    {
        $prefix = config('database.connections.mysql.prefix');

        $builder = self::select('shimmer_liveshop_goods.*')
            ->distinct("{$prefix}shimmer_liveshop_goods.id")
            ->leftJoin('shimmer_liveshop_relation_tag_xrefs', 'shimmer_liveshop_relation_tag_xrefs.relation_id', 'shimmer_liveshop_goods.id')->where('relation_type', 1)
            ->withoutGlobalScope('uniacid')
            ->where('shimmer_liveshop_goods.uniacid', AppService::$uniacid)
            ->where('tag_id', $tagId)
            ->where('shimmer_liveshop_goods.state', 1)
            ->orderBy('shimmer_liveshop_goods.sort', 'desc');

        if ($limit) {
            $builder->limit($limit);
        }

        return $builder->get();
    }
}
